from __future__ import print_function
from __future__ import absolute_import

__author__ = 'Gramazio Kohler Research'
__copyright__ = 'Copyright 2020 Gramazio Kohler Research'
__license__ = 'MIT'
__email__ = 'gramaziokohler@arch.ethz.ch'

__version__ = '0.3.1'

__all__ = ['__author__', '__email__', '__copyright__', '__license__', '__version__']
